package labWork.premium_store.model;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
public class Tank {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @Column(nullable = false)
    private String name;

    @ManyToOne
    @JoinColumn(name = "nation_id")
    private Nation nation;

    @ManyToOne
    @JoinColumn(name = "level_id")
    private Level level;

    @Column(nullable = false)
    private int cost;

    //реализация двунаправленной связи мнокие-ко-многим
    @ManyToMany
    @JoinTable(name = "tanks_of_clients",
            joinColumns = @JoinColumn(name = "tank_fk"),
            inverseJoinColumns = @JoinColumn(name = "client_fk"))
    private List<Client> clients;

    public Tank() {
    }

    public Tank(String name, Nation nation, Level level, int cost) {
        this.name = name;
        this.nation = nation;
        this.level = level;
        this.cost = cost;
    }

    public Long getId(){
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getCost() {
        return cost;
    }

    public void setCost(int cost) {
        this.cost = cost;
    }

    public Level getLevel() {
        return level;
    }

    public void setLevel(Level level) {
        this.level = level;
    }

    public Nation getNation() {
        return nation;
    }

    public void setNation(Nation nation) {
        this.nation = nation;
    }

    public List<Client> getClients() {
        return clients;
    }

    public void setClients(List<Client> clients) {
        this.clients = clients;
    }

    public void addClient(Client newClient){
        if  (clients == null){
            this.clients = new ArrayList<>();
        }

        if (!clients.contains(newClient))
            this.clients.add(newClient);
    }

    public void removeClient(Client remClient){
        if (clients.contains(remClient))
            this.clients.remove(remClient);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;

        if (!(o instanceof Tank tank)) return false;

        return Objects.equals(getId(), tank.getId()) && Objects.equals(getName(), tank.getName())
                && Objects.equals(getCost(), tank.getCost()) && Objects.equals(getClients(), tank.getClients())
                && Objects.equals(getLevel(), tank.getLevel());
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    //преобразование данных по объекту в строчку
    @Override
    public String toString() {
        return "Student{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", nation='" + nation + '\'' +
                ", level='" + level + '\'' +
                ", cost='" + cost + '\'' +
                '}';
    }
}
