package labWork.premium_store.service;

import labWork.premium_store.model.Nation;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;

@Service
public class NationService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Nation addNation(String nameNation){
        if(!StringUtils.hasText(nameNation)){
            throw new IllegalArgumentException("The name of the nation is null or empty");
        }
        
        final Nation nation = new Nation(nameNation);
        em.persist(nation);
        
        return nation;
    }

    @Transactional(readOnly = true)
    public Nation findNation(Long id){
        final Nation nation = em.find(Nation.class, id);

        if(nation == null){
            throw new EntityNotFoundException(String.format("Nation with id [%s] is not found", id));
        }

        return nation;
    }

    @Transactional(readOnly = true)
    public List<Nation> findAllNations() {
        return em.createQuery("select n from Nation n", Nation.class)
                .getResultList();
    }

    @Transactional
    public Nation updateNation(Long id, String newNation) {
        if (!StringUtils.hasText(newNation)) {
            throw new IllegalArgumentException("Student name is null or empty");
        }

        final Nation currentNation = findNation(id);
        currentNation.setNation(newNation);

        return em.merge(currentNation);
    }

    @Transactional
    public Nation deleteNAtion(Long id) {
        final Nation currentNation = findNation(id);
        em.remove(currentNation);

        return currentNation;
    }

    @Transactional
    public void deleteAllNations() {
        em.createQuery("delete from Nation").executeUpdate();
    }
}
