package labWork.premium_store.service;

import labWork.premium_store.model.Client;
import labWork.premium_store.model.Level;
import labWork.premium_store.model.Nation;
import labWork.premium_store.model.Tank;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;

@Service
public class TankService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Tank addTank(String newName, Nation newNation, Level newLevel, int newCost) {
        if (!StringUtils.hasText(newName) || newNation == null || newLevel == null || newCost <= 0) {
            throw new IllegalArgumentException("New Name, new Nation, new Level or newCost is zero or empty");
        }

        final Tank tank = new Tank(newName, newNation, newLevel, newCost);
        em.persist(tank);

        return tank;
    }

    @Transactional(readOnly = true)
    public Tank findTank(Long id) {
        final Tank Tank = em.find(Tank.class, id);

        if (Tank == null) {
            throw new EntityNotFoundException(String.format("Tank with id [%s] is not found", id));
        }

        return Tank;
    }

    @Transactional(readOnly = true)
    public List<Tank> findAllTanks() {
        return em.createQuery("select t from Tank t", Tank.class)
                .getResultList();
    }

    @Transactional
    public Tank updateTank(Long id, String newName, Nation newNation,
                           Level newLevel, int newCost, List<Client> newClients) {
        if (id <= 0) {
            throw new IllegalArgumentException("Invalid id");
        }

        final Tank currentTank = findTank(id);

        if (StringUtils.hasText(newName)){
            currentTank.setName(newName);
        }

        if(newNation != null){
            currentTank.setNation(newNation);
        }

        if(newLevel != null){
            currentTank.setLevel(newLevel);
        }

        if(newCost > 0){
            currentTank.setCost(newCost);
        }

        if(newClients != null){
            currentTank.setClients(newClients);
        }

        return em.merge(currentTank);
    }

    @Transactional
    public Tank deleteTank(Long id) {
        final Tank currentTank = findTank(id);
        em.remove(currentTank);

        return currentTank;
    }

    //прямой sql-запрос на удаление всех записей в таблице
    @Transactional
    public void deleteAllTanks() {
        em.createQuery("Delete from Tank").executeUpdate();
    }
}
